## Dr. Frinks Server Controller
## Usage:
## 	Easy Way
## 		python kagserv.py
##	Interactive way
##		python
##		>> from kagserv import *
##	Recommended Interactive way
##		ipython
##		In [0]: from kagserv import *
##
## Please send any significant changes to sprucegum@gmail.com

from subprocess import Popen, PIPE, call
from time import time
from threading import Timer
from os import getcwd
from sys import exit
from signal import signal, SIGINT

## Settings:

MEMLIMIT = 250 		# megabytes, restart the server even if occupied if it hits this RAM limit.
RESTART_PERIOD = 20 	# minutes, minimum time between restarts 
POLL_PERIOD = 10	# seconds, how often to see how many people are playing, perform logic.
UPDATE_PERIOD = 30	# minutes, how often to check for updates.

class KagServer(object):
	def __init__(self):
		self.sPID = self.getPID()
		if (self.sPID == -1):
			self.KAG = self.start_server()
		else:
			self.KAG = self.attach_server()
		self.run_manager = True
		self.timer = None
		self.restarts = 0
		self.memrestarts = 0
		self.updates = 0

	def get_players(self):
		with open('Logs/stats.txt','r') as stats:
			return int(stats.read().split()[3])

	def start_server(self):
		self.start_time = time() 
		self.run_manager = True
		self.fresh = True
		self.last_update_check = time() - UPDATE_PERIOD*60
		return Popen(['./KAGdedi', \
		'autostart Scripts/dedicated_autostart.gm ',\
		'autoconfig Scripts/dedicated_autoconfig.gm'])

	def attach_server(self):
		self.start_time = time() - RESTART_PERIOD*60
		self.run_manager = True
		self.fresh = False 
		self.last_update_check = time() - UPDATE_PERIOD*60
		return ProxyProcess(self.sPID)

	def getPID(self):
		try:
                	return int(Popen(["ps","-C","KAGdedi","-o","pid"],stdout=PIPE).stdout.read().split()[1])
		except:
			return -1

	def kill_server(self):	
		self.timer.cancel()
		self.run_manager = False
		self.KAG.terminate()
	
	def restart_server(self):
		self.restarts += 1
		self.kill_server()
		self.KAG = self.start_server()

	def get_uptime(self):
		return (time() - self.start_time)

	def running(self):
		self.KAG.poll()
		return (self.KAG.returncode is None)
	
	def get_mem(self):
		ps = Popen(['ps','vh','-p', '{0}'.format(self.KAG.pid)],stdout=PIPE)
		return int(ps.stdout.read().split()[6])

	def manager(self,restart_period = 60*RESTART_PERIOD, poll_period = POLL_PERIOD, memory_limit = MEMLIMIT*1024):
			if self.run_manager:			
				t = Timer(poll_period, self.manager)
				t.start()
				players = self.get_players()
				#print("Players:{0}".format(players))
				if not self.running():
					self.KAG = self.start_server()
				
				elif (self.get_mem()> memory_limit):
					self.memrestart += 1
					self.restart_server()
				elif ((players > 0) and self.fresh):
					self.fresh = False
				elif ((players == 0) and not self.fresh):
					if ((time() - self.start_time)>restart_period):
						print("Restarting server after {0} hours gameplay".format(self.get_uptime()/3600.0))
						self.restart_server()
				elif (((time() - self.last_update_check)/60) > UPDATE_PERIOD):
					if (players == 0):
						print("Checking for Update")
						if self.check_update():
							self.updates += 1
							self.restart_server()

	def state(self):
		print("Players:{0} Uptime:{1:.2}h Mem:{2:.2}m Restarts:{3}".format(\
			self.get_players(),\
			self.get_uptime()/3600,\
			self.get_mem()/1024.0,\
			self.restarts))	

	def check_update(self):
		self.last_update_check = time()
		with open('App/version.txt') as this_version_file:
			this_version = this_version_file.readline().split()[1]
		call(['wget','-q','-N','http://update.kag2d.com/kag_linux/App/version.txt'])
		with open('version.txt') as current_version_file:
			current_version = current_version_file.readline().split()[1]
		return (current_version != this_version)

	def ctrlc(self, signum,frame):
		self.kill_server()
		print("exiting")
		exit()	
	
class ProxyProcess(object):
	def __init__(self, PID):
		self.pid = PID
		self.returncode = None

	def poll(self):
		try:
			self.pid = int(Popen(["ps","-C","KAGdedi","-o","pid"],stdout=PIPE).stdout.read().split()[1])
                	return None
		except:
			return 0

	def terminate(self):
		call(['kill',str(self.pid)])

## Autostart

K = KagServer()
K.timer = Timer(30,K.manager)
K.timer.start()
signal(SIGINT, K.ctrlc)


